@@ -1,8 +1,9 @@
 from onegov.file.collection import FileCollection, FileSetCollection
 from onegov.file.integration import DepotApp
-from onegov.file.models import File, FileSet
+from onegov.file.models import File, FileSet, AssociatedFiles
 
 __all__ = [
+    'AssociatedFiles',
     'DepotApp',
     'File',
     'FileCollection',
