@@ -1,11 +1,16 @@
 import morepath
 import os.path
+import shlex
+import shutil
+import subprocess
 
+from blinker import ANY
 from contextlib import contextmanager
 from depot.manager import DepotManager
 from depot.middleware import FileServeApp
 from more.transaction.main import transaction_tween_factory
 from morepath import App
+from onegov.core.custom import json
 from onegov.core.security import Private, Public
 from onegov.file.collection import FileCollection
 from onegov.file.models import File
@@ -43,20 +48,69 @@ class DepotApp(App):
             file belongs to which application. Additionally it ensures that we
             aren't accidentally opening another application's files.
 
+        :frontend_cache_buster: A script able to bust the frontend cache.
+
+            Our frontend (nginx) caches the files we store in the backend and
+            serves them mostly without bothering us. This can be problematic
+            when the file is deleted or if it is made private. The cache
+            needs to be busted in this case.
+
+            With this configuration a script/command can be specified that
+            receives the url that needs to be busted and in turn busts the
+            content of this url from the cache. This pretty much depends
+            on the platform this is run and on the frontend in use.
+
+            For example, let's say our script is called 'bust-cache', this
+            is the command that will be run when the cache is busted::
+
+                sleep 5
+                bust-cache id-of-the-file
+
+            As you can see, the command is invoked with a five second delay.
+            This avoids premature cache busting (before the end of the
+            transaction). The command is non-blocking, so those 5 seconds
+            are not counted towards the request-time.
+
+            Frontend caches might use the domain and the full path to cache
+            a file, but since we can technically have multiple domains/paths
+            point to the same file we simply pass the id and let the cache
+            figure out what urls need to be busted as a result.
+
+            The script is invoked with the permissions of the user running
+            the backend. If other permissions are required, use suid.
+
+            Note that this script is optional. If omitted, the cache busting
+            turns into a noop.
+
         """
 
         self.depot_backend = cfg.get('depot_backend')
         self.depot_storage_path = cfg.get('depot_storage_path')
 
+        self.frontend_cache_buster = cfg.get('frontend_cache_buster')
+        self.frontend_cache_bust_delay = cfg.get(
+            'frontend_cache_bust_delay', 5)
+
         assert self.depot_backend in SUPPORTED_STORAGE_BACKENDS, """
             A depot app *must* have a valid storage backend set up.
         """
 
+        if not shutil.which('gs'):
+            raise RuntimeError("onegov.file requires ghostscript")
+
         if self.depot_backend == 'depot.io.local.LocalFileStorage':
             assert os.path.isdir(self.depot_storage_path), """
                 The depot storage path must exist.
             """
 
+        if self.frontend_cache_buster:
+
+            @self.session_manager.on_update.connect_via(ANY, weak=False)
+            @self.session_manager.on_delete.connect_via(ANY, weak=False)
+            def on_file_change(schema, obj):
+                if isinstance(obj, File):
+                    self.bust_frontend_cache(obj.id)
+
     @property
     def bound_depot_id(self):
         return self.custom_depot_id or self.schema
@@ -99,6 +153,16 @@ class DepotApp(App):
 
         DepotManager.set_default(self.bound_depot_id)
 
+    def bust_frontend_cache(self, file_id):
+        if not self.frontend_cache_buster:
+            return
+
+        bin = shlex.quote(self.frontend_cache_buster)
+        fid = shlex.quote(file_id)
+        cmd = f'sleep {self.frontend_cache_bust_delay} && {bin} {fid}'
+
+        subprocess.Popen(cmd, close_fds=True, shell=True)
+
     def clear_depot_cache(self):
         DepotManager._aliases.clear()
         DepotManager._default_depot = None
@@ -147,7 +211,18 @@ def render_depot_file(file, request):
 def respond_with_alt_text(reference, request):
     @request.after
     def include_alt_text(response):
-        response.headers.add('X-File-Note', reference.note or '')
+        # HTTP headers are limited to ASCII, so we encode our result in
+        # JSON before showing it
+        response.headers.add('X-File-Note', json.dumps(
+            {'note': reference.note or ''},
+        ))
+
+
+def respond_with_caching_header(reference, request):
+    if not reference.published:
+        @request.after
+        def include_private_header(response):
+            response.headers.add('Cache-Control', 'private')
 
 
 @DepotApp.path(model=File, path='/storage/{id}')
@@ -158,17 +233,26 @@ def get_file(app, id):
 @DepotApp.view(model=File, render=render_depot_file, permission=Public)
 def view_file(self, request):
     respond_with_alt_text(self, request)
+    respond_with_caching_header(self, request)
     return self.reference.file
 
 
-@DepotApp.view(model=File, name='thumbnail', render=render_depot_file,
-               permission=Public)
+@DepotApp.view(model=File, name='thumbnail', permission=Public,
+               render=render_depot_file)
+@DepotApp.view(model=File, name='small', permission=Public,
+               render=render_depot_file)
+@DepotApp.view(model=File, name='medium', permission=Public,
+               render=render_depot_file)
 def view_thumbnail(self, request):
-    respond_with_alt_text(self, request)
+    if request.view_name in ('small', 'medium'):
+        size = request.view_name
+    else:
+        size = 'small'
 
-    # we currently only have one thumbnail, in the future we might make this
-    # a query parameter:
-    thumbnail_id = self.get_thumbnail_id(size='small')
+    respond_with_alt_text(self, request)
+    respond_with_caching_header(self, request)
+
+    thumbnail_id = self.get_thumbnail_id(size)
 
     if not thumbnail_id:
         return morepath.redirect(request.link(self))
@@ -212,6 +296,20 @@ def handle_note_update(self, request):
         flag_modified(self, 'modified')
 
 
+@DepotApp.view(model=File, name='rename', request_method='POST',
+               permission=Private)
+def handle_rename(self, request):
+    request.assert_valid_csrf_token()
+    self.name = request.POST.get('name')
+
+    # when updating the name we offer the option not to update the
+    # modified date, which is helpful if the files are in modified order
+    # and the order should remain when the note is changed
+    if request.POST.get('keep-timestamp') in ('1', 'true', 'yes'):
+        self.modified = self.modified
+        flag_modified(self, 'modified')
+
+
 @DepotApp.view(model=File, request_method='DELETE', permission=Private)
 def delete_file(self, request):
     """ Deletes the given file. By default the permission is
@@ -229,4 +327,4 @@ def delete_file(self, request):
 
     """
     request.assert_valid_csrf_token()
-    FileCollection(request.app.session()).delete(self)
+    FileCollection(request.session).delete(self)
