@@ -1,28 +1,36 @@
 from depot.fields.sqlalchemy import UploadedFileField as UploadedFileFieldBase
 from onegov.core.crypto import random_token
 from onegov.core.orm import Base
+from onegov.core.orm.abstract import Associable
 from onegov.core.orm.mixins import TimestampMixin
-from onegov.file.filters import OnlyIfImage, WithThumbnailFilter
+from onegov.core.orm.types import JSON, UTCDateTime
+from onegov.core.utils import normalize_for_url
 from onegov.file.attachments import ProcessedUploadedFile
-from sqlalchemy import Column, Text
+from onegov.file.filters import OnlyIfImage, WithThumbnailFilter
+from onegov.file.filters import OnlyIfPDF, WithPDFThumbnailFilter
+from sqlalchemy import Boolean, Column, Index, Text
 from sqlalchemy_utils import observes
 
 
 class UploadedFileField(UploadedFileFieldBase):
     """ A customized version of Depot's uploaded file field. This version
-    stores its data in an unlimited text field, whereas the original uses
-    a limited varchar field.
-
-    We probably won't hit the 4000 character limit any time soon, but to
-    avoid any chance of this happening, we use Text instead.
+    stores its data in a JSONB field, instead of using text.
 
     """
 
     def load_dialect_impl(self, dialect):
-        return dialect.type_descriptor(Text())
+        return dialect.type_descriptor(JSON())
+
+    def process_bind_param(self, value, dialect):
+        if value:
+            return value
+
+    def process_result_value(self, value, dialect):
+        if value:
+            return self._upload_type(value)
 
 
-class File(Base, TimestampMixin):
+class File(Base, Associable, TimestampMixin):
     """ A general file (image, document, pdf, etc), referenced in the database.
 
     Thanks to the use of `Depot <http://depot.readthedocs.io>`_ files
@@ -42,6 +50,21 @@ class File(Base, TimestampMixin):
     #: a short note about the file (for captions, other information)
     note = Column(Text, nullable=True)
 
+    #: the default order of files
+    order = Column(Text, nullable=False)
+
+    #: true if published
+    published = Column(Boolean, nullable=False, default=True)
+
+    #: the date after which this file will be made public - this controls
+    #: the visibility of the object through the ``is_hidden_from_public``
+    #: property which in turn is enforced by :mod:`onegov.core.security.rules`.
+    #:
+    #: To get a file published, be sure to call
+    #: :meth:`onegov.file.collection.FileCollection.publish_files` once an
+    #: hour through a cronjob (see :mod:`onegov.core.cronjobs`)!
+    publish_date = Column(UTCDateTime, nullable=True)
+
     #: the type of the file, this can be used to create custom polymorphic
     #: subclasses. See `<http://docs.sqlalchemy.org/en/improve_toc/
     #: orm/extensions/declarative/inheritance.html>`_.
@@ -61,6 +84,11 @@ class File(Base, TimestampMixin):
                 WithThumbnailFilter(
                     name='small', size=(256, 256), format='png'
                 )
+            ),
+            OnlyIfPDF(
+                WithPDFThumbnailFilter(
+                    name='medium', size=(512, 512), format='png'
+                )
             )
         ]
     ), nullable=False)
@@ -78,10 +106,22 @@ class File(Base, TimestampMixin):
         'polymorphic_on': 'type'
     }
 
+    __table_args__ = (
+        Index('files_by_type_and_order', 'type', 'order'),
+    )
+
     @observes('reference')
     def reference_observer(self, reference):
         self.checksum = self.reference.get('checksum')
 
+    @observes('name')
+    def name_observer(self, name):
+        self.order = normalize_for_url(name)
+
+    @property
+    def is_hidden_from_public(self):
+        return not self.published
+
     @property
     def file_id(self):
         """ The file_id of the contained reference.
