@@ -1,17 +1,30 @@
 import transaction
 import pytest
 
+from depot.manager import DepotManager
 from io import BytesIO
-from onegov.file import File, FileSet
+from onegov.core.orm import Base
+from onegov.core.utils import module_path
+from onegov.file import File, FileSet, AssociatedFiles
 from onegov.file.models.fileset import file_to_set_associations
-from onegov.testing.utils import create_image
+from onegov_testing.utils import create_image
 from PIL import Image
+from sqlalchemy import Column
+from sqlalchemy import Integer
+from sqlalchemy import Text
 
 
 class PolymorphicFile(File):
     __mapper_args__ = {'polymorphic_identity': 'polymorphic'}
 
 
+class Blogpost(Base, AssociatedFiles):
+    __tablename__ = 'blogposts'
+
+    id = Column(Integer, primary_key=True)
+    text = Column(Text, nullable=False)
+
+
 @pytest.mark.parametrize('cls', [File, PolymorphicFile])
 def test_store_file_from_string(session, cls):
     session.add(cls(name='readme.txt', reference=b'README\n======'))
@@ -116,6 +129,27 @@ def test_thumbnail_creation(session):
     assert 'thumbnail_small' in small.reference
 
 
+def test_pdf_preview_creation(session):
+    path = module_path('onegov.file', 'tests/fixtures/example.pdf')
+
+    with open(path, 'rb') as f:
+        session.add(File(name='example.pdf', reference=f))
+
+    transaction.commit()
+
+    pdf = session.query(File).one()
+    pdf.reference['thumbnail_medium']
+
+    # our example file contains a blue backgorund so we can verify that
+    # the thumbnail for the pdf is generated correctly
+
+    thumb = DepotManager.get().get(pdf.reference['thumbnail_medium']['id'])
+    image = Image.open(thumb)
+
+    assert (0, 91, 161, 255) in set(image.getdata())
+    assert image.size == (395, 512)
+
+
 def test_max_image_size(session):
     session.add(File(name='unchanged.png', reference=create_image(1024, 1024)))
     session.add(File(name='limited.png', reference=create_image(1025, 1024)))
@@ -184,3 +218,25 @@ def test_determine_unknown_svg_size(session, temporary_path):
     # use the default max size as the size if we can't determine one
     logo = session.query(File).order_by(File.name).one()
     assert logo.reference.size == ['1024px', '1024px']
+
+
+def test_associated_files(session):
+    post = Blogpost(
+        text="My interview at <company>",
+        files=[File(name='frowney.png', reference=create_image(1024, 1024))]
+    )
+
+    session.add(post)
+    session.flush()
+
+    assert len(post.files) == 1
+    assert post.files[0].name == 'frowney.png'
+    assert post.files[0].reference.size == ('1024px', '1024px')
+
+    assert session.query(File).one().linked_blogposts == [post]
+    assert session.query(File).one().links.all() == (post, )
+
+    session.delete(post)
+    session.flush()
+
+    assert session.query(File).count() == 0
